@@ -21,8 +21,8 @@
 #include <KXmlGuiWindow>
 
 class GameEngine;
-//class Scene; 
 class CanvasWidget;
+class KToggleAction;
 
 class MainWindow : public KXmlGuiWindow
 {
@@ -37,9 +37,19 @@ private slots:
     void showHighscores();
     void loadSettings();
     void startNewGame();
+    void fire();
+    // calls the action
+    void pauseGame();
+    // called by the action
+    void setGamePaused(bool paused);
+    // assures the action not checked anymore
+    void handleGameResumed();
+    void viewFullScreen(bool fullScreen);
     
 private:
     void setupActions();
+
+    KToggleAction *pauseAction;
     
     CanvasWidget *canvasWidget;
     GameEngine *gameEngine;
